@@ -10,7 +10,7 @@ import multiprocessing as _multiprocessing
 import itertools as _itertools
 
 
-__version__ = '0.0.18'
+__version__ = '0.0.19'
 
 
 class Error(Exception):
@@ -20,6 +20,51 @@ class Error(Exception):
 TICK_INTERVAL_PADDING_RATIO = 0.1
 
 
+def binning(xs, bins):
+    if bins < 1:
+        return []
+    n_xs = len(xs)
+    if n_xs < 1:
+        return []
+    elif n_xs == 1:
+        x_min = xs[0] - 1/2
+        x_max = xs[0] + 1/2
+    else:
+        x_min, x_max = min_max(xs)
+    dx = (x_max - x_min)/bins
+    assert 0 < max(abs(x_min), abs(x_max))*_sys.float_info.epsilon <= dx
+    ns = [0 for _ in range(bins)]
+    for x in xs:
+        fi_bin = (x - x_min)/dx
+        i_bin = int(fi_bin)
+        if fi_bin != i_bin:
+            ns[i_bin] += 1
+        elif i_bin <= 0:
+            ns[0] += 1
+        elif i_bin >= bins:
+            ns[-1] += 1
+        else:
+            ns[i_bin] += 1/2
+            ns[i_bin - 1] += 1/2
+    return [{'x1': x1,
+             'x2': x2,
+             'n': n,
+             'y': n/n_xs/dx}
+            for (x1, x2), n
+            in zip(each_cons(linspace(x_min, x_max, bins + 1), 2), ns)]
+
+
+def min_max(xs):
+    assert xs
+    min_ = max_ = xs[0]
+    for x in xs[1:]:
+        if x > max_:
+            max_ = x
+        elif x < min_:
+            min_ = x
+    return min_, max_
+
+
 def linspace(start, stop, num=50):
     if num < 1:
         return []
@@ -346,6 +391,25 @@ def _fn_for_test_parallel_for(x, y):
 
 class _Tester(_unittest.TestCase):
 
+    def test_binning(self):
+        bins = 10
+        bs = binning([0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10], bins)
+        b0 = bs[0]
+        self.assertAlmostEqual(b0['x1'], 0)
+        self.assertAlmostEqual(b0['x2'], 1)
+        self.assertAlmostEqual(b0['n'], 1.5)
+        self.assertAlmostEqual(b0['y'], 1.5/11)
+        bbins = bs[-1]
+        self.assertAlmostEqual(bbins['x1'], 9)
+        self.assertAlmostEqual(bbins['x2'], 10)
+        self.assertAlmostEqual(bbins['n'], 1.5)
+        self.assertAlmostEqual(bbins['y'], 1.5/11)
+        for i, b in enumerate(bs[1:bins-1]):
+            self.assertAlmostEqual(b['x1'], i + 1)
+            self.assertAlmostEqual(b['x2'], i + 2)
+            self.assertAlmostEqual(b['n'], 1)
+            self.assertAlmostEqual(b['y'], 1/11)
+
     def test_linspace(self):
         for x, y in zip(linspace(0, 10, 11), list(range(11))):
             self.assertAlmostEqual(x, y)
